// Copyright (C) 2020 Alexis Jeandet.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0

#pragma once

#include "versionhelper.h"

namespace MesonProjectManager {
namespace Internal {

struct MesonInfo
{
    Version mesonVersion;
};

} // namespace Internal
} // namespace MesonProjectManager
