// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0

#pragma once

#include <coreplugin/dialogs/ioptionspage.h>

namespace Core {
namespace Internal {

class LocatorSettingsPage : public IOptionsPage
{
    Q_OBJECT

public:
    LocatorSettingsPage();
};

} // namespace Internal
} // namespace Core
