// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0

#pragma once

namespace Bookmarks {
namespace Constants {

const char BOOKMARKS_TOGGLE_ACTION[]      = "Bookmarks.Toggle";
const char BOOKMARKS_EDIT_ACTION[]        = "Bookmarks.Edit";
const char BOOKMARKS_MOVEUP_ACTION[]      = "Bookmarks.MoveUp";
const char BOOKMARKS_MOVEDOWN_ACTION[]    = "Bookmarks.MoveDown";
const char BOOKMARKS_EDITNOTE_ACTION[]    = "Bookmarks.EditNote";
const char BOOKMARKS_PREV_ACTION[]        = "Bookmarks.Previous";
const char BOOKMARKS_NEXT_ACTION[]        = "Bookmarks.Next";
const char BOOKMARKS_PREVDIR_ACTION[]     = "Bookmarks.PreviousDirectory";
const char BOOKMARKS_NEXTDIR_ACTION[]     = "Bookmarks.NextDirectory";
const char BOOKMARKS_PREVDOC_ACTION[]     = "Bookmarks.PreviousDocument";
const char BOOKMARKS_NEXTDOC_ACTION[]     = "Bookmarks.NextDocument";
const char BOOKMARKS_TEXT_MARK_CATEGORY[] = "Bookmarks.TextMarkCategory";

const char BOOKMARKS_MENU[]               = "Bookmarks.Menu";
const char BOOKMARKS_CONTEXT[]            = "Bookmarks";

} // namespace Constants
} // namespace Bookmarks
