// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0

#pragma once

namespace Core {
namespace Internal {

class LoggingViewer
{
public:
    static void showLoggingView();
};

} // Internal
} // Core
