// Copyright (C) 2020 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0

#pragma once

#include <projectexplorer/buildstep.h>

namespace IncrediBuild {
namespace Internal {

class BuildConsoleStepFactory final : public ProjectExplorer::BuildStepFactory
{
public:
    BuildConsoleStepFactory();
};

} // namespace Internal
} // namespace IncrediBuild
